@@ -1,31 +1,31 @@
-branch <- "cran/v0.7.2"
+branch <- "cran/v0.8.0"
 
 install_config <- list(
-  "1.10.2" = list(
+  "1.11.0" = list(
     "cpu" = list(
       "darwin" = list(
         "libtorch" = list(
-          url = "https://download.pytorch.org/libtorch/cpu/libtorch-macos-1.10.2.zip",
+          url = "https://download.pytorch.org/libtorch/cpu/libtorch-macos-1.11.0.zip",
           path = "libtorch/",
           filter = ".dylib",
-          md5hash = "96ebbf1e2e44f30ee80bf3c8e4a31e15"
+          md5hash = "5b5c2e96d7a944cb014dec46e2f06b82"
         ),
         "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/macOS-cpu.zip", branch)
       ),
       "windows" = list(
         "libtorch" = list(
-          url = "https://download.pytorch.org/libtorch/cpu/libtorch-win-shared-with-deps-1.10.2%2Bcpu.zip",
+          url = "https://download.pytorch.org/libtorch/cpu/libtorch-win-shared-with-deps-1.11.0%2Bcpu.zip",
           path = "libtorch/",
           filter = ".dll",
-          md5hash = "c49ddfd07ba65e0ff4a54e041ed22c42"
+          md5hash = "4a54e26cc90eb7035957a6b364d2f4ca"
         ),
         "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Windows-cpu.zip", branch)
       ),
       "linux" = list(
         "libtorch" = list(
           path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cpu/libtorch-cxx11-abi-shared-with-deps-1.10.2%2Bcpu.zip",
-          md5hash = "99d16043865716f5e38a8d15480b61c6"
+          url = "https://download.pytorch.org/libtorch/cpu/libtorch-cxx11-abi-shared-with-deps-1.11.0%2Bcpu.zip",
+          md5hash = "85aae2aaf309472f39037febebd7f978"
         ),
         "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Linux-cpu.zip", branch)
       )
@@ -34,55 +34,27 @@ install_config <- list(
       "linux" = list(
         "libtorch" = list(
           path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cu102/libtorch-cxx11-abi-shared-with-deps-1.10.2%2Bcu102.zip",
-          md5hash = "ea32cd0be31f75078208fd484b3982e8"
+          url = "https://download.pytorch.org/libtorch/cu102/libtorch-cxx11-abi-shared-with-deps-1.11.0%2Bcu102.zip",
+          md5hash = "e3033206012923b1e4b0e5e686b8b62a"
         ),
         "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Linux-gpu-102.zip", branch)
-      ),
-      "windows" = list(
-        "libtorch" = list(
-          path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cu102/libtorch-win-shared-with-deps-1.10.2%2Bcu102.zip",
-          filter = ".dll",
-          md5hash = "8a3496640497cbec833e7fc12ae91c94"
-        ),
-        "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Windows-gpu-102.zip", branch)
-      )
-    ),
-    "11.1" = list(
-      "linux" = list(
-        "libtorch" = list(
-          path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cu111/libtorch-cxx11-abi-shared-with-deps-1.10.2%2Bcu111.zip",
-          md5hash = "5e0afdd052fa25d150c2ee5bdb64e6fb"
-        ),
-        "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Linux-gpu-111.zip", branch)
-      ),
-      "windows" = list(
-        "libtorch" = list(
-          path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cu111/libtorch-win-shared-with-deps-1.10.2%2Bcu111.zip",
-          filter = ".dll",
-          md5hash = "23d56f370a306e9d480a8d60513d9e67"
-        ),
-        "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Windows-gpu-111.zip", branch)
       )
     ),
     "11.3" = list(
       "linux" = list(
         "libtorch" = list(
           path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cu113/libtorch-cxx11-abi-shared-with-deps-1.10.2%2Bcu113.zip",
-          md5hash = "b7e28b3d8edb8ebb7d5428f0c8fb928d"
+          url = "https://download.pytorch.org/libtorch/cu113/libtorch-cxx11-abi-shared-with-deps-1.11.0%2Bcu113.zip",
+          md5hash = "f7c14a977d7842d1079f3cda6d010ba0"
         ),
         "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Linux-gpu-113.zip", branch)
       ),
       "windows" = list(
         "libtorch" = list(
           path = "libtorch/",
-          url = "https://download.pytorch.org/libtorch/cu113/libtorch-win-shared-with-deps-1.10.2%2Bcu113.zip",
+          url = "https://download.pytorch.org/libtorch/cu113/libtorch-win-shared-with-deps-1.11.0%2Bcu113.zip",
           filter = ".dll",
-          md5hash = "d3711140709c64764db837040921b19f"
+          md5hash = "be7a27312c797a04f2870f9a41cd6589"
         ),
         "liblantern" = sprintf("https://storage.googleapis.com/torch-lantern-builds/refs/heads/%s/latest/Windows-gpu-113.zip", branch)
       )
@@ -90,7 +62,7 @@ install_config <- list(
   )
 )
 
-install_path <- function(version = "1.10.2") {
+install_path <- function(version = "1.11.0") {
   path <- Sys.getenv("TORCH_HOME")
   if (nzchar(path)) {
     normalizePath(path, mustWork = FALSE)
@@ -246,6 +218,12 @@ install_type_windows <- function(version) {
     }
   }
 
+  # Query nvcc from cuda in cuda_path.
+  if (nzchar(cuda_path) && is.null(cuda_version)) {
+    nvcc_path <- file.path(cuda_path, "bin", "nvcc.exe")
+    cuda_version <- nvcc_version_from_path(nvcc_path)
+  }
+
   if (is.null(cuda_version)) {
     return("cpu")
   }
@@ -357,7 +335,7 @@ install_type <- function(version) {
 #' reported length, an increase of the \code{timeout} value should help.
 #'
 #' @export
-install_torch <- function(version = "1.10.2", type = install_type(version = version), reinstall = FALSE,
+install_torch <- function(version = "1.11.0", type = install_type(version = version), reinstall = FALSE,
                           path = install_path(), timeout = 360, ...) {
   if (reinstall) {
     unlink(path, recursive = TRUE)
@@ -416,7 +394,7 @@ install_torch <- function(version = "1.10.2", type = install_type(version = vers
 #'
 #'
 #' @export
-install_torch_from_file <- function(version = "1.10.2", type = install_type(version = version), libtorch, liblantern, ...) {
+install_torch_from_file <- function(version = "1.11.0", type = install_type(version = version), libtorch, liblantern, ...) {
   stopifnot(inherits(url(libtorch), "file"))
   stopifnot(inherits(url(liblantern), "file"))
 
@@ -435,7 +413,7 @@ install_torch_from_file <- function(version = "1.10.2", type = install_type(vers
 #'
 #'
 #' @export
-get_install_libs_url <- function(version = "1.10.2", type = install_type(version = version)) {
+get_install_libs_url <- function(version = "1.11.0", type = install_type(version = version)) {
   libtorch <- install_config[[version]][[type]][[install_os()]][["libtorch"]][["url"]]
   liblantern <- install_config[[version]][[type]][[install_os()]][["liblantern"]]
   list(libtorch = libtorch, liblantern = liblantern)
